import { RabbitMqBroker, RedisConnector } from 'ts-common';
import { REDIS_PASSWORD, REDIS_HOST, REDIS_PORT, RABBIT_USER, RABBIT_PASSWORD, RABBIT_PORT, RABBIT_HOST } from './variables';
import { log } from './logger';
import { queryServiceReader } from './readers/queryService';


async function main() {
    log.info('Starting query-service...');
    const frontendPublisher = new RabbitMqBroker('ui-direct-exchange', "query-service");
    const mlPublisher = new RabbitMqBroker('ml-direct-exchange', "query-service", undefined, { autoDelete: false }, { autoDelete: false });
    log.info('Connecting to Redis...');
    const redis = new RedisConnector(REDIS_PASSWORD, REDIS_HOST, REDIS_PORT);
    log.info('Connecting to RabbitMQ...');

    await redis.connect();

    log.info('Connected to Redis!');
    await frontendPublisher.connect({
        protocol: 'amqp',
        hostname: RABBIT_HOST,
        port: RABBIT_PORT,
        username: RABBIT_USER,
        password: RABBIT_PASSWORD,
    });
    await mlPublisher.connect({
        protocol: 'amqp',
        hostname: RABBIT_HOST,
        port: RABBIT_PORT,
        username: RABBIT_USER,
        password: RABBIT_PASSWORD,
    });
    log.info('Connected to RabbitMQ!');

    await queryServiceReader(frontendPublisher, mlPublisher, redis, 'neo4j');
    // TODO: other query services for other databases

    log.info('Connected to RabbitMQ!');
}

main();