import type { AnyStatement } from "ts-common/src/model/query/logic/general";
import type { QueryCacheData } from "./model";

export function createWhereLogic(op: string, left: string, whereLogic: string, cacheData: QueryCacheData): [string, string] {
    const newWhereLogic = `${left.replace(".", "_")}_${op}`;
    if (whereLogic) {
        whereLogic += ", ";
    }
    const remainingNodes: string[] = [];

    for (const entity of cacheData.entities) {
        if (entity.id !== left) {
            remainingNodes.push(entity.id);
        }
    }

    // TODO: Relation temporarily ignored due to unnecessary added complexity in the query
    // for (const relation of cacheData.relations) {
    //     if (relation.id !== left) {
    //         remainingNodes.push(relation.id);
    //     }
    // }

    const remainingNodesStr = remainingNodes.length > 0 ? `, ${remainingNodes.join(", ")}` : "";
    const newWithLogic = `${whereLogic}${op}(${left}) AS ${newWhereLogic}${remainingNodesStr}`;

    return [newWhereLogic, newWithLogic];
}

export function extractLogicCypher(logicQuery: AnyStatement, cacheData: QueryCacheData): [string, string] {
    switch (typeof logicQuery) {
        case "object":
            if (Array.isArray(logicQuery)) {
                let op = logicQuery[0].replace("_", "").toLowerCase();
                let [left, whereLogic] = extractLogicCypher(logicQuery[1], cacheData);

                switch (op) {
                    case "!=":
                        op = "<>";
                        break;
                    case "==":
                        op = "=";
                        break;
                    case "like":
                        op = "=~";
                        break;
                    case "isempty":
                        return [`(${left} IS NULL OR ${left} = "")`, whereLogic];
                    case "lower":
                        return [`toLower(${left})`, whereLogic];
                    case "upper":
                        return [`toUpper(${left})`, whereLogic];
                    case "avg":
                    case "count":
                    case "max":
                    case "min":
                    case "sum":
                        return createWhereLogic(op, left, whereLogic, cacheData);
                }
                if (logicQuery.length > 2 && logicQuery[2]) {
                    let [right, whereLogicRight] = extractLogicCypher(logicQuery[2], cacheData);

                    if (whereLogicRight) {
                        if (whereLogic) {
                            whereLogic += ", ";
                        }
                        whereLogic += whereLogicRight;
                    }
                    if (op === "=~") {
                        right = `(".*" + ${right} + ".*")`;
                    }
                    return [`(${left} ${op} ${right})`, whereLogic];
                }
                return [`(${op} ${left})`, whereLogic];
            }
            return [logicQuery, ""];
        case "string":
            return [logicQuery.replace("@", ""), ""];
        case "number":
            return [logicQuery.toString(), ""];
        default:
            return [logicQuery as any, ""];
    }
}
